@@ -19,6 +19,7 @@ from sushy import connector as sushy_connector
 from sushy import exceptions
 from sushy.resources import base
 from sushy.resources.manager import manager
+from sushy.resources.registry import message_registry_file
 from sushy.resources.sessionservice import session
 from sushy.resources.sessionservice import sessionservice
 from sushy.resources.system import system
@@ -26,6 +27,24 @@ from sushy.resources.system import system
 LOG = logging.getLogger(__name__)
 
 
+class ProtocolFeaturesSupportedField(base.CompositeField):
+
+    excerpt_query = base.Field('ExcerptQuery')
+    """The excerpt query parameter is supported"""
+
+    expand_query = base.Field('ExpandQuery')
+    """The expand query parameter is supported"""
+
+    filter_query = base.Field('FilterQuery')
+    """The filter query parameter is supported"""
+
+    only_member_query = base.Field('OnlyMemberQuery')
+    """The only query parameter is supported"""
+
+    select_query = base.Field('SelectQuery')
+    """The select query parameter is supported"""
+
+
 class Sushy(base.ResourceBase):
 
     identity = base.Field('Id', required=True)
@@ -37,6 +56,13 @@ class Sushy(base.ResourceBase):
     uuid = base.Field('UUID')
     """The Redfish root service UUID"""
 
+    product = base.Field('Product')
+    """The product associated with this Redfish service"""
+
+    protocol_features_supported = ProtocolFeaturesSupportedField(
+        'ProtocolFeaturesSupported')
+    """The information about protocol features supported by the service"""
+
     _systems_path = base.Field(['Systems', '@odata.id'])
     """SystemCollection path"""
 
@@ -46,6 +72,9 @@ class Sushy(base.ResourceBase):
     _session_service_path = base.Field(['SessionService', '@odata.id'])
     """SessionService path"""
 
+    _registries_path = base.Field(['Registries', '@odata.id'])
+    """Registries path"""
+
     def __init__(self, base_url, username=None, password=None,
                  root_prefix='/redfish/v1/', verify=True,
                  auth=None, connector=None):
@@ -81,10 +110,21 @@ class Sushy(base.ResourceBase):
         super(Sushy, self).__init__(
             connector or sushy_connector.Connector(base_url, verify=verify),
             path=self._root_prefix)
+        self._base_url = base_url
         self._auth = auth
         self._auth.set_context(self, self._conn)
         self._auth.authenticate()
 
+    def __del__(self):
+        if self._auth:
+            try:
+                self._auth.close()
+
+            except Exception as ex:
+                LOG.warning('Ignoring error while closing Redfish session '
+                            'with %s: %s', self._base_url, ex)
+            self._auth = None
+
     def _parse_attributes(self):
         super(Sushy, self)._parse_attributes()
         self.redfish_version = self.json.get('RedfishVersion')
@@ -158,3 +198,18 @@ class Sushy(base.ResourceBase):
         """
         return session.Session(self._conn, identity,
                                redfish_version=self.redfish_version)
+
+    def _get_registry_collection(self):
+        """Get MessageRegistryFileCollection object
+
+        This resource is optional and can be empty.
+
+        :returns: MessageRegistryFileCollection object
+            or None if Registries not provided
+        """
+
+        if self._registries_path:
+            return message_registry_file.MessageRegistryFileCollection(
+                self._conn,
+                self._registries_path,
+                redfish_version=self.redfish_version)
