@@ -187,6 +187,39 @@ class ListField(Field):
         return instances
 
 
+class DictionaryField(Field):
+    """Base class for fields consisting of dictionary of several sub-fields."""
+
+    def __init__(self, *args, **kwargs):
+        super(DictionaryField, self).__init__(*args, **kwargs)
+        self._subfields = dict(_collect_fields(self))
+
+    def _load(self, body, resource, nested_in=None):
+        """Load the dictionary.
+
+        :param body: parent JSON body.
+        :param resource: parent resource.
+        :param nested_in: parent resource name (for error reporting only).
+        :returns: a new dictionary object containing subfields.
+        """
+        nested_in = (nested_in or []) + self._path
+        values = super(DictionaryField, self)._load(body, resource)
+        if values is None:
+            return None
+
+        instances = {}
+        for key, value in values.items():
+            instance_value = copy.copy(self)
+            for attr, field in self._subfields.items():
+                # Hide the Field object behind the real value
+                setattr(instance_value, attr, field._load(value,
+                                                          resource,
+                                                          nested_in))
+            instances[key] = instance_value
+
+        return instances
+
+
 class MappedField(Field):
     """Field taking real value from a mapping."""
 
@@ -283,12 +316,20 @@ class ResourceBase(object):
         resource specific refresh operations to be performed. This is a
         primitive method in the paradigm of Template design pattern.
 
+        As for the base implementation of this method the approach taken is:
+        On refresh, all sub-resources are marked as stale. That means
+        invalidate (or undefine) the exposed attributes for nested resources
+        for fresh evaluation in subsequent calls to those exposed attributes.
+        In other words greedy-refresh is not done for them, unless forced by
+        ``force`` argument.
+
         :param force: should force refresh the resource and its sub-resources,
             if set to True.
         :raises: ResourceNotFoundError
         :raises: ConnectionError
         :raises: HTTPError
         """
+        utils.cache_clear(self, force_refresh=force)
 
     def invalidate(self, force_refresh=False):
         """Mark the resource as stale, prompting refresh() before getting used.
@@ -360,6 +401,7 @@ class ResourceCollectionBase(ResourceBase):
         return self._resource_type(self._conn, identity,
                                    redfish_version=self.redfish_version)
 
+    @utils.cache_it
     def get_members(self):
         """Return a list of ``_resource_type`` objects present in collection
 
