@@ -23,6 +23,8 @@ from sushy.resources.system import constants as sys_cons
 from sushy.resources.system import ethernet_interface
 from sushy.resources.system import mappings as sys_maps
 from sushy.resources.system import processor
+from sushy.resources.system import simple_storage as sys_simple_storage
+from sushy.resources.system.storage import storage as sys_storage
 from sushy import utils
 
 
@@ -72,7 +74,7 @@ class System(base.ResourceBase):
     """The system BIOS version"""
 
     boot = BootField('Boot', required=True)
-    """A dictionary containg the current boot device, frequency and mode"""
+    """A dictionary containing the current boot device, frequency and mode"""
 
     description = base.Field('Description')
     """The system description"""
@@ -119,14 +121,8 @@ class System(base.ResourceBase):
     memory_summary = MemorySummaryField('MemorySummary')
     """The summary info of memory of the system in general detail"""
 
-    _processors = None  # ref to ProcessorCollection instance
-
     _actions = ActionsField('Actions', required=True)
 
-    _ethernet_interfaces = None
-
-    _bios = None
-
     def __init__(self, connector, identity, redfish_version=None):
         """A class representing a ComputerSystem
 
@@ -253,6 +249,7 @@ class System(base.ResourceBase):
         return utils.get_sub_resource_path_by(self, 'Processors')
 
     @property
+    @utils.cache_it
     def processors(self):
         """Property to reference `ProcessorCollection` instance
 
@@ -260,15 +257,12 @@ class System(base.ResourceBase):
         this property is marked as stale (greedy-refresh not done).
         Here the actual refresh of the sub-resource happens, if stale.
         """
-        if self._processors is None:
-            self._processors = processor.ProcessorCollection(
-                self._conn, self._get_processor_collection_path(),
-                redfish_version=self.redfish_version)
-
-        self._processors.refresh(force=False)
-        return self._processors
+        return processor.ProcessorCollection(
+            self._conn, self._get_processor_collection_path(),
+            redfish_version=self.redfish_version)
 
     @property
+    @utils.cache_it
     def ethernet_interfaces(self):
         """Property to reference `EthernetInterfaceCollection` instance
 
@@ -276,17 +270,13 @@ class System(base.ResourceBase):
         this property is marked as stale (greedy-refresh not done).
         Here the actual refresh of the sub-resource happens, if stale.
         """
-        if self._ethernet_interfaces is None:
-            self._ethernet_interfaces = (
-                ethernet_interface.EthernetInterfaceCollection(
-                    self._conn,
-                    utils.get_sub_resource_path_by(self, "EthernetInterfaces"),
-                    redfish_version=self.redfish_version))
-
-        self._ethernet_interfaces.refresh(force=False)
-        return self._ethernet_interfaces
+        return ethernet_interface.EthernetInterfaceCollection(
+            self._conn,
+            utils.get_sub_resource_path_by(self, "EthernetInterfaces"),
+            redfish_version=self.redfish_version)
 
     @property
+    @utils.cache_it
     def bios(self):
         """Property to reference `Bios` instance
 
@@ -294,28 +284,53 @@ class System(base.ResourceBase):
         this property is marked as stale (greedy-refresh not done).
         Here the actual refresh of the sub-resource happens, if stale.
         """
-        if self._bios is None:
-            self._bios = bios.Bios(
-                self._conn,
-                utils.get_sub_resource_path_by(self, 'Bios'),
-                redfish_version=self.redfish_version)
+        return bios.Bios(
+            self._conn,
+            utils.get_sub_resource_path_by(self, 'Bios'),
+            redfish_version=self.redfish_version)
 
-        self._bios.refresh(force=False)
-        return self._bios
+    @property
+    @utils.cache_it
+    def simple_storage(self):
+        """A collection of simple storage associated with system.
 
-    def _do_refresh(self, force=False):
-        """Do custom resource specific refresh activities
+        This returns a reference to `SimpleStorageCollection` instance.
+        SimpleStorage represents the properties of a storage controller and its
+        directly-attached devices.
 
-        On refresh, all sub-resources are marked as stale, i.e.
-        greedy-refresh not done for them unless forced by ``force``
-        argument.
+        It is set once when the first time it is queried. On refresh,
+        this property is marked as stale (greedy-refresh not done).
+        Here the actual refresh of the sub-resource happens, if stale.
+
+        :raises: MissingAttributeError if 'SimpleStorage/@odata.id' field
+            is missing.
+        :returns: `SimpleStorageCollection` instance
         """
-        if self._processors is not None:
-            self._processors.invalidate(force)
-        if self._ethernet_interfaces is not None:
-            self._ethernet_interfaces.invalidate(force)
-        if self._bios is not None:
-            self._bios.invalidate(force)
+        return sys_simple_storage.SimpleStorageCollection(
+            self._conn, utils.get_sub_resource_path_by(self, "SimpleStorage"),
+            redfish_version=self.redfish_version)
+
+    @property
+    @utils.cache_it
+    def storage(self):
+        """A collection of storage subsystems associated with system.
+
+        This returns a reference to `StorageCollection` instance.
+        A storage subsystem represents a set of storage controllers (physical
+        or virtual) and the resources such as drives and volumes that can be
+        accessed from that subsystem.
+
+        It is set once when the first time it is queried. On refresh,
+        this property is marked as stale (greedy-refresh not done).
+        Here the actual refresh of the sub-resource happens, if stale.
+
+        :raises: MissingAttributeError if 'Storage/@odata.id' field
+            is missing.
+        :returns: `StorageCollection` instance
+        """
+        return sys_storage.StorageCollection(
+            self._conn, utils.get_sub_resource_path_by(self, "Storage"),
+            redfish_version=self.redfish_version)
 
 
 class SystemCollection(base.ResourceCollectionBase):
