@@ -19,10 +19,12 @@ import mock
 
 import sushy
 from sushy import exceptions
-from sushy.resources.system import constants as sys_cons
-from sushy.resources.system import ethernet_interface
+from sushy.resources import constants as res_cons
+from sushy.resources.system import bios
 from sushy.resources.system import mappings as sys_map
 from sushy.resources.system import processor
+from sushy.resources.system import simple_storage
+from sushy.resources.system.storage import storage
 from sushy.resources.system import system
 from sushy.tests.unit import base
 
@@ -32,8 +34,8 @@ class SystemTestCase(base.TestCase):
     def setUp(self):
         super(SystemTestCase, self).setUp()
         self.conn = mock.Mock()
-        with open('sushy/tests/unit/json_samples/system.json', 'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/system.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
 
         self.sys_inst = system.System(
             self.conn, '/redfish/v1/Systems/437XR1138R2',
@@ -57,15 +59,14 @@ class SystemTestCase(base.TestCase):
         self.assertEqual('Physical', self.sys_inst.system_type)
         self.assertEqual('38947555-7742-3448-3784-823347823834',
                          self.sys_inst.uuid)
-        self.assertEqual('Enabled', self.sys_inst.status.state)
-        self.assertEqual('OK', self.sys_inst.status.health)
-        self.assertEqual('OK', self.sys_inst.status.health_rollup)
+        self.assertEqual(res_cons.STATE_ENABLED, self.sys_inst.status.state)
+        self.assertEqual(res_cons.HEALTH_OK, self.sys_inst.status.health)
+        self.assertEqual(res_cons.HEALTH_OK,
+                         self.sys_inst.status.health_rollup)
         self.assertEqual(sushy.SYSTEM_POWER_STATE_ON,
                          self.sys_inst.power_state)
         self.assertEqual(96, self.sys_inst.memory_summary.size_gib)
         self.assertEqual("OK", self.sys_inst.memory_summary.health)
-        self.assertIsNone(self.sys_inst._processors)
-        self.assertIsNone(self.sys_inst._ethernet_interfaces)
 
     def test__parse_attributes_missing_actions(self):
         self.sys_inst.json.pop('Actions')
@@ -265,13 +266,11 @@ class SystemTestCase(base.TestCase):
         self.assertIsNone(self.sys_inst.memory_summary)
 
     def test_processors(self):
-        # check for the underneath variable value
-        self.assertIsNone(self.sys_inst._processors)
         # | GIVEN |
         self.conn.get.return_value.json.reset_mock()
-        with open('sushy/tests/unit/json_samples/processor_collection.json',
-                  'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/'
+                  'processor_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
         # | WHEN |
         actual_processors = self.sys_inst.processors
         # | THEN |
@@ -289,47 +288,43 @@ class SystemTestCase(base.TestCase):
 
     def test_processors_on_refresh(self):
         # | GIVEN |
-        with open('sushy/tests/unit/json_samples/processor_collection.json',
-                  'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/'
+                  'processor_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
         # | WHEN & THEN |
         self.assertIsInstance(self.sys_inst.processors,
                               processor.ProcessorCollection)
 
         # On refreshing the system instance...
-        with open('sushy/tests/unit/json_samples/system.json', 'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/system.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
 
         self.sys_inst.invalidate()
         self.sys_inst.refresh(force=False)
 
-        # | WHEN & THEN |
-        self.assertIsNotNone(self.sys_inst._processors)
-        self.assertTrue(self.sys_inst._processors._is_stale)
-
         # | GIVEN |
-        with open('sushy/tests/unit/json_samples/processor_collection.json',
-                  'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/'
+                  'processor_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
         # | WHEN & THEN |
         self.assertIsInstance(self.sys_inst.processors,
                               processor.ProcessorCollection)
-        self.assertFalse(self.sys_inst._processors._is_stale)
 
     def _setUp_processor_summary(self):
         self.conn.get.return_value.json.reset_mock()
-        with open('sushy/tests/unit/json_samples/processor_collection.json',
-                  'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/'
+                  'processor_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
 
         # fetch processors for the first time
         self.sys_inst.processors
 
         successive_return_values = []
-        with open('sushy/tests/unit/json_samples/processor.json', 'r') as f:
-            successive_return_values.append(json.loads(f.read()))
-        with open('sushy/tests/unit/json_samples/processor2.json', 'r') as f:
-            successive_return_values.append(json.loads(f.read()))
+        file_names = ['sushy/tests/unit/json_samples/processor.json',
+                      'sushy/tests/unit/json_samples/processor2.json']
+        for file_name in file_names:
+            with open(file_name) as f:
+                successive_return_values.append(json.load(f))
 
         self.conn.get.return_value.json.side_effect = successive_return_values
 
@@ -358,26 +353,130 @@ class SystemTestCase(base.TestCase):
         self.conn.get.return_value.json.reset_mock()
         eth_coll_return_value = None
         eth_return_value = None
-        path = ('sushy/tests/unit/json_samples/'
-                'ethernet_interfaces_collection.json')
-        with open(path, 'r') as f:
-            eth_coll_return_value = json.loads(f.read())
-        with open('sushy/tests/unit/json_samples/ethernet_interfaces.json',
-                  'r') as f:
-            eth_return_value = (json.loads(f.read()))
+        with open('sushy/tests/unit/json_samples/'
+                  'ethernet_interfaces_collection.json') as f:
+            eth_coll_return_value = json.load(f)
+        with open('sushy/tests/unit/json_samples/'
+                  'ethernet_interfaces.json') as f:
+            eth_return_value = json.load(f)
 
         self.conn.get.return_value.json.side_effect = [eth_coll_return_value,
                                                        eth_return_value]
 
-        self.assertIsNone(self.sys_inst._ethernet_interfaces)
         actual_macs = self.sys_inst.ethernet_interfaces.summary
         expected_macs = (
-            {'12:44:6A:3B:04:11':
-             sys_map.HEALTH_STATE_VALUE_MAP_REV.get(
-                 sys_cons.HEALTH_STATE_ENABLED)})
+            {'12:44:6A:3B:04:11': res_cons.STATE_ENABLED})
         self.assertEqual(expected_macs, actual_macs)
-        self.assertIsInstance(self.sys_inst._ethernet_interfaces,
-                              ethernet_interface.EthernetInterfaceCollection)
+
+    def test_bios(self):
+        self.conn.get.return_value.json.reset_mock()
+        bios_return_value = None
+        with open('sushy/tests/unit/json_samples/bios.json') as f:
+            bios_return_value = json.load(f)
+        self.conn.get.return_value.json.side_effect = [bios_return_value]
+
+        self.assertIsInstance(self.sys_inst.bios, bios.Bios)
+        self.assertEqual('BIOS Configuration Current Settings',
+                         self.sys_inst.bios.name)
+
+    def test_simple_storage_for_missing_attr(self):
+        self.sys_inst.json.pop('SimpleStorage')
+        with self.assertRaisesRegex(
+                exceptions.MissingAttributeError, 'attribute SimpleStorage'):
+            self.sys_inst.simple_storage
+
+    def test_simple_storage(self):
+        # | GIVEN |
+        self.conn.get.return_value.json.reset_mock()
+        with open('sushy/tests/unit/json_samples/'
+                  'simple_storage_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+        # | WHEN |
+        actual_simple_storage = self.sys_inst.simple_storage
+        # | THEN |
+        self.assertIsInstance(actual_simple_storage,
+                              simple_storage.SimpleStorageCollection)
+        self.conn.get.return_value.json.assert_called_once_with()
+
+        # reset mock
+        self.conn.get.return_value.json.reset_mock()
+        # | WHEN & THEN |
+        # tests for same object on invoking subsequently
+        self.assertIs(actual_simple_storage,
+                      self.sys_inst.simple_storage)
+        self.conn.get.return_value.json.assert_not_called()
+
+    def test_simple_storage_on_refresh(self):
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'simple_storage_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+        # | WHEN & THEN |
+        self.assertIsInstance(self.sys_inst.simple_storage,
+                              simple_storage.SimpleStorageCollection)
+
+        # On refreshing the system instance...
+        with open('sushy/tests/unit/json_samples/system.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        self.sys_inst.invalidate()
+        self.sys_inst.refresh(force=False)
+
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'simple_storage_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+        # | WHEN & THEN |
+        self.assertIsInstance(self.sys_inst.simple_storage,
+                              simple_storage.SimpleStorageCollection)
+
+    def test_storage_for_missing_attr(self):
+        self.sys_inst.json.pop('Storage')
+        with self.assertRaisesRegex(
+                exceptions.MissingAttributeError, 'attribute Storage'):
+            self.sys_inst.storage
+
+    def test_storage(self):
+        # | GIVEN |
+        self.conn.get.return_value.json.reset_mock()
+        with open('sushy/tests/unit/json_samples/'
+                  'storage_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+        # | WHEN |
+        actual_storage = self.sys_inst.storage
+        # | THEN |
+        self.assertIsInstance(actual_storage, storage.StorageCollection)
+        self.conn.get.return_value.json.assert_called_once_with()
+
+        # reset mock
+        self.conn.get.return_value.json.reset_mock()
+        # | WHEN & THEN |
+        # tests for same object on invoking subsequently
+        self.assertIs(actual_storage, self.sys_inst.storage)
+        self.conn.get.return_value.json.assert_not_called()
+
+    def test_storage_on_refresh(self):
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'storage_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+        # | WHEN & THEN |
+        self.assertIsInstance(self.sys_inst.storage,
+                              storage.StorageCollection)
+
+        # On refreshing the system instance...
+        with open('sushy/tests/unit/json_samples/system.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+
+        self.sys_inst.invalidate()
+        self.sys_inst.refresh(force=False)
+
+        # | GIVEN |
+        with open('sushy/tests/unit/json_samples/'
+                  'storage_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
+        # | WHEN & THEN |
+        self.assertIsInstance(self.sys_inst.storage, storage.StorageCollection)
 
 
 class SystemCollectionTestCase(base.TestCase):
@@ -386,8 +485,8 @@ class SystemCollectionTestCase(base.TestCase):
         super(SystemCollectionTestCase, self).setUp()
         self.conn = mock.Mock()
         with open('sushy/tests/unit/json_samples/'
-                  'system_collection.json', 'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+                  'system_collection.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
         self.sys_col = system.SystemCollection(
             self.conn, '/redfish/v1/Systems', redfish_version='1.0.2')
 
