@@ -36,19 +36,35 @@ class ResourceBaseTestCase(base.TestCase):
         self.conn = mock.Mock()
         self.base_resource = BaseResource(connector=self.conn, path='/Foo',
                                           redfish_version='1.0.2')
+        self.assertFalse(self.base_resource._is_stale)
         # refresh() is called in the constructor
         self.conn.reset_mock()
 
-    def test_refresh(self):
+    def test_refresh_no_force(self):
+        self.base_resource.refresh(force=False)
+        self.conn.get.assert_not_called()
+
+    def test_refresh_force(self):
         self.base_resource.refresh()
         self.conn.get.assert_called_once_with(path='/Foo')
 
+    def test_invalidate(self):
+        self.base_resource.invalidate()
+        self.conn.get.assert_not_called()
+
+        self.base_resource.refresh(force=False)
+        self.conn.get.assert_called_once_with(path='/Foo')
+
+    def test_invalidate_force_refresh(self):
+        self.base_resource.invalidate(force_refresh=True)
+        self.conn.get.assert_called_once_with(path='/Foo')
+
 
 class TestResource(resource_base.ResourceBase):
     """A concrete Test Resource to test against"""
 
     def __init__(self, connector, identity, redfish_version=None):
-        """Ctor of TestResouce
+        """Ctor of TestResource
 
         :param connector: A Connector instance
         :param identity: The id of the Resource
@@ -113,10 +129,9 @@ class ResourceCollectionBaseTestCase(base.TestCase):
                           self.test_resource_collection.get_member, '2')
         self.conn.get.assert_called_once_with(path='Fakes/2')
 
-    def test_get_members(self):
+    def _validate_get_members_result(self, member_ids):
         # | GIVEN |
         # setting some valid member paths
-        member_ids = ('1', '2')
         self.test_resource_collection.members_identities = member_ids
         # | WHEN |
         result = self.test_resource_collection.get_members()
@@ -126,6 +141,45 @@ class ResourceCollectionBaseTestCase(base.TestCase):
             self.assertIsInstance(val, TestResource)
             self.assertTrue(val.identity in member_ids)
             self.assertEqual('1.0.x', val.redfish_version)
+            self.assertFalse(val._is_stale)
+
+        return result
+
+    def test_get_members(self):
+        self._validate_get_members_result(('1', '2'))
+
+    def test_get_members_on_refresh(self):
+        all_members = self._validate_get_members_result(('1', '2'))
+
+        # Call resource invalidate
+        self.test_resource_collection.invalidate()
+        self.assertTrue(self.test_resource_collection._is_stale)
+        #  Now invoke refresh action on resource. This can be viewed as
+        # "light refresh" which involves only the resource's fresh retrieval
+        # and not its nested resources (these are only marked as stale).
+        self.test_resource_collection.refresh(force=False)
+        # resource itself is fresh
+        self.assertFalse(self.test_resource_collection._is_stale)
+        # members are marked as stale
+        for m in all_members:
+            self.assertTrue(m._is_stale)
+
+        self._validate_get_members_result(('1', '2'))
+        # members are also now freshly retrieved
+        for m in all_members:
+            self.assertFalse(m._is_stale)
+
+        # Again invalidate and do a forced refresh on resource
+        self.test_resource_collection.invalidate(force_refresh=True)
+        # Now, even the members are also freshly retrieved. This can be viewed
+        # as "cascading refresh" which involves not only the resource's fresh
+        # retrieval but also its nested resources.
+        for m in all_members:
+            self.assertFalse(m._is_stale)
+
+    def test_get_members_caching(self):
+        result = self._validate_get_members_result(('1', '2'))
+        self.assertIs(result, self.test_resource_collection.get_members())
 
 
 TEST_JSON = {
@@ -139,6 +193,20 @@ TEST_JSON = {
             'Field': 'field value'
         },
         'Mapped': 'raw'
+    },
+    'ListField': [
+        {
+            'String': 'a third string',
+            'Integer': 1
+        },
+        {
+            'String': 'a fourth string',
+            'Integer': 2
+        }
+    ],
+    'Dictionary': {
+        'key1': {'property_a': 'value1', 'property_b': 'value2'},
+        'key2': {'property_a': 'value3', 'property_b': 'value4'}
     }
 }
 
@@ -156,10 +224,22 @@ class NestedTestField(resource_base.CompositeField):
     non_existing = resource_base.Field('NonExisting', default=3.14)
 
 
+class TestListField(resource_base.ListField):
+    string = resource_base.Field('String', required=True)
+    integer = resource_base.Field('Integer', adapter=int)
+
+
+class TestDictionaryField(resource_base.DictionaryField):
+    property_a = resource_base.Field('property_a')
+    property_b = resource_base.Field('property_b')
+
+
 class ComplexResource(resource_base.ResourceBase):
     string = resource_base.Field('String', required=True)
     integer = resource_base.Field('Integer', adapter=int)
     nested = NestedTestField('Nested')
+    field_list = TestListField('ListField')
+    dictionary = TestDictionaryField('Dictionary')
     non_existing_nested = NestedTestField('NonExistingNested')
     non_existing_mapped = resource_base.MappedField('NonExistingMapped',
                                                     MAPPING)
@@ -182,42 +262,52 @@ class FieldTestCase(base.TestCase):
         self.assertEqual('field value', self.test_resource.nested.nested_field)
         self.assertEqual('real', self.test_resource.nested.mapped)
         self.assertEqual(3.14, self.test_resource.nested.non_existing)
+        self.assertEqual('a third string',
+                         self.test_resource.field_list[0].string)
+        self.assertEqual(2, self.test_resource.field_list[1].integer)
+        self.assertEqual(2, len(self.test_resource.dictionary))
+        self.assertEqual('value1',
+                         self.test_resource.dictionary['key1'].property_a)
+        self.assertEqual('value4',
+                         self.test_resource.dictionary['key2'].property_b)
         self.assertIsNone(self.test_resource.non_existing_nested)
         self.assertIsNone(self.test_resource.non_existing_mapped)
 
     def test_missing_required(self):
         del self.json['String']
-        self.assertRaisesRegex(exceptions.MissingAttributeError,
-                               'String', self.test_resource.refresh)
+        self.assertRaisesRegex(
+            exceptions.MissingAttributeError,
+            'String', self.test_resource.refresh, force=True)
 
     def test_missing_nested_required(self):
         del self.json['Nested']['String']
-        self.assertRaisesRegex(exceptions.MissingAttributeError,
-                               'Nested/String', self.test_resource.refresh)
+        self.assertRaisesRegex(
+            exceptions.MissingAttributeError,
+            'Nested/String', self.test_resource.refresh, force=True)
 
     def test_missing_nested_required2(self):
         del self.json['Nested']['Object']['Field']
         self.assertRaisesRegex(exceptions.MissingAttributeError,
                                'Nested/Object/Field',
-                               self.test_resource.refresh)
+                               self.test_resource.refresh, force=True)
 
     def test_malformed_int(self):
         self.json['Integer'] = 'banana'
         self.assertRaisesRegex(
             exceptions.MalformedAttributeError,
             'attribute Integer is malformed.*invalid literal for int',
-            self.test_resource.refresh)
+            self.test_resource.refresh, force=True)
 
     def test_malformed_nested_int(self):
         self.json['Nested']['Integer'] = 'banana'
         self.assertRaisesRegex(
             exceptions.MalformedAttributeError,
             'attribute Nested/Integer is malformed.*invalid literal for int',
-            self.test_resource.refresh)
+            self.test_resource.refresh, force=True)
 
     def test_mapping_missing(self):
         self.json['Nested']['Mapped'] = 'banana'
-        self.test_resource.refresh()
+        self.test_resource.refresh(force=True)
 
         self.assertIsNone(self.test_resource.nested.mapped)
 
