@@ -22,6 +22,7 @@ from sushy import connector
 from sushy import exceptions
 from sushy import main
 from sushy.resources.manager import manager
+from sushy.resources.registry import message_registry_file
 from sushy.resources.sessionservice import session
 from sushy.resources.sessionservice import sessionservice
 from sushy.resources.system import system
@@ -40,8 +41,8 @@ class MainTestCase(base.TestCase):
         self.sess_serv.create_session.return_value = (None, None)
         mock_session_service.return_value = self.sess_serv
         mock_connector.return_value = self.conn
-        with open('sushy/tests/unit/json_samples/root.json', 'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+        with open('sushy/tests/unit/json_samples/root.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
         self.root = main.Sushy('http://foo.bar:1234',
                                verify=True, auth=mock_auth)
         mock_connector.assert_called_once_with(
@@ -54,6 +55,13 @@ class MainTestCase(base.TestCase):
         self.assertEqual('1.0.2', self.root.redfish_version)
         self.assertEqual('92384634-2938-2342-8820-489239905423',
                          self.root.uuid)
+        self.assertEqual('Product', self.root.product)
+        self.assertTrue(self.root.protocol_features_supported.excerpt_query)
+        self.assertFalse(self.root.protocol_features_supported.expand_query)
+        self.assertTrue(self.root.protocol_features_supported.filter_query)
+        self.assertTrue(
+            self.root.protocol_features_supported.only_member_query)
+        self.assertFalse(self.root.protocol_features_supported.select_query)
         self.assertEqual('/redfish/v1/Systems', self.root._systems_path)
         self.assertEqual('/redfish/v1/Managers', self.root._managers_path)
         self.assertEqual('/redfish/v1/SessionService',
@@ -68,9 +76,9 @@ class MainTestCase(base.TestCase):
     @mock.patch.object(connector, 'Connector', autospec=True)
     def test_custom_connector(self, mock_Sushy_Connector):
         connector_mock = mock.MagicMock()
-        with open('sushy/tests/unit/json_samples/root.json', 'r') as f:
+        with open('sushy/tests/unit/json_samples/root.json') as f:
             connector_mock.get.return_value.json.return_value = (
-                json.loads(f.read()))
+                json.load(f))
         main.Sushy('http://foo.bar:1234', 'foo', 'bar',
                    connector=connector_mock)
         self.assertTrue(connector_mock.post.called)
@@ -119,6 +127,16 @@ class MainTestCase(base.TestCase):
             self.root._conn, 'asdf',
             redfish_version=self.root.redfish_version)
 
+    @mock.patch.object(message_registry_file,
+                       'MessageRegistryFileCollection',
+                       autospec=True)
+    def test__get_registry_collection(
+        self, MessageRegistryFileCollection_mock):
+        self.root._get_registry_collection()
+        MessageRegistryFileCollection_mock.assert_called_once_with(
+            self.root._conn, '/redfish/v1/Registries',
+            redfish_version=self.root.redfish_version)
+
 
 class BareMinimumMainTestCase(base.TestCase):
 
@@ -126,8 +144,8 @@ class BareMinimumMainTestCase(base.TestCase):
         super(BareMinimumMainTestCase, self).setUp()
         self.conn = mock.MagicMock()
         with open('sushy/tests/unit/json_samples/'
-                  'bare_minimum_root.json', 'r') as f:
-            self.conn.get.return_value.json.return_value = json.loads(f.read())
+                  'bare_minimum_root.json') as f:
+            self.conn.get.return_value.json.return_value = json.load(f)
         self.root = main.Sushy('http://foo.bar:1234', verify=True,
                                auth=mock.MagicMock(), connector=self.conn)
 
@@ -145,3 +163,6 @@ class BareMinimumMainTestCase(base.TestCase):
         self.assertRaisesRegex(
             exceptions.MissingAttributeError,
             'SessionService/@odata.id', self.root.get_session_service)
+
+    def test__get_registry_collection_when_registries_attr_absent(self):
+        self.assertIsNone(self.root._get_registry_collection())
